/**
 * @class ComponentDescriptor
 *
 * @description (write description)
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function ComponentDescriptor(){
    
    var self = this;
    
    /** Represent if descriptor object can change actions
     * @private
     * @type {Boolean}
     * @default 0
     */
    var m_bEditable = false;
    
    /** More than action can be choice
     * @private
     * @type {Boolean}
     * @default true
     */
    var m_bActionEditable = true;
    
    /** Value can be edit
     * @private
     * @type {Boolean}
     * @default true
     */
    var m_bValueEditable = true;
    
    /** Default value
     * @private
     * @type {Integer}
     * @default 0
     */
    //var m_nDefaultValue = 0;
     
    /** Default value
     * @private
     * @type {Integer}
     * @default 0
     */
    var m_nValue = 0;
    
    /** Default action
     * @private
     * @type {Integer}
     * @default 0
     */
    var m_iAction = 0;
    
    /** Min value
     * @private
     * @type {Integer}
     * @default 0
     */
    var m_nMin = 0;
    
    /** Max value
     * @private
     * @type {Integer}
     * @default 0
     */
    var m_nMax = 0;
    
    /** Type of value (Int = 0; Float = 1)
     * @private
     * @type {Integer}
     * @default 0
     */
    var m_iType = 0;
    
    
    //var m_oDurationAction = undefined;                                        
    
    /** Duration (scenario composition version 2)
     * @private
     * @type Integer
     * @default undefined
     */
    var m_iDuration = undefined;
    
    /** Description data
     * @private
     * @type object
     * @default new Object()
     */
    var m_oMap = new Object();

    var m_ADescriptions=new Array();
    /** Get nex free id
     * @param {Integer} iType Type (Integer = 0; Float = 1)
     * @param {Number} nMin Min value
     * @param {Number} nMax Max value
     * @param {Number} nVal Default value
     * @returns {Integer} Error type (Ok = 0, Error = otherwise)
     */
    this.setValueDescription = function(iType,nMin,nMax,nVal){
        m_iType = iType;
        m_nMin = nMin;
        m_nMax = nMax;
        
        //if(nVal < m_nMin || nVal > m_nMax){ console.error("Default value is not inside range ["+m_nMin+", "+m_nMax+"]"); return 1; }
        m_nValue = nVal;
        return 0;
    };
    
    /** Add description to object
     * @param {Integer} Description enumeration
     * @param {string} Description
     */
    this.addDescription = function(val,strDescription){
        console.log({"value":val,"description":strDescription});
        m_ADescriptions.push({"value":val,"description":strDescription});
     };
    
    /** Remove description to m_oMap object
     * @param {Integer} Key of area map
     */
    this.removeDescriptionByVal = function(val){

        for (var i=0;i<m_ADescriptions.length;i++)
        {
            if (m_ADescriptions[i].value==val) {
                m_ADescriptions.splice(i, 1);
            }
        }

    };

    this.getIndexByVal = function(val){

        for (var i=0;i<m_ADescriptions.length;i++)
        {
            console.log("Val:" +m_ADescriptions[i].value);
            if (m_ADescriptions[i].value==val) {
                return i;
            }
        }
        return -1;
    };


    /** Set default action
     * @param {Integer} iAction Default action
     */
    this.setDefaultAction = function(iAction){
        m_iAction = iAction;
    };
    
    /** Get default action
     * @returns {Integer} m_iAction Default action
     */
    this.getDefaultAction = function(){
        return m_iAction;
    };
    
    /** Get description by value
     * @returns {string} Action description
     */
    this.getDescriptionByValue = function(val){
        var oMap=new object();
        for (var i=0;i<m_ADescriptions.length;i++)
        {
            if (m_ADescriptions[i].value==val) {
                return m_ADescriptions[i].description;
            }
        }
        return "";
    };

    this.getArrayDescriptions = function(val){
        return m_ADescriptions;
    };



    /** Get default action
     * @returns {Integer} Default value
     */
    //this.getDefaultAction = function(){ return m_nValue; };
    
    /** Set editable
     * @param {Boolean} bIsEditable Boolean value for describe if value field is editable or not
     */
    this.setEditable = function(bIsEditable){
        m_bEditable = bIsEditable;
    };
    
    /** Get editable
     * @returns {Boolean} bIsEditable Boolean value for describe if value field is editable or not
     */
    this.isEditable = function(){
        return m_bEditable;
    };
    
    /** Set action editable
     * @param {Boolean} bEditable Boolean value
     */
    this.setActionEditable = function(bEditable){
        m_bActionEditable = bEditable;
    };
    
    /** Get action editable
     * @returns {Boolean} bIsEditable Boolean value for describe if value field is editable or not
     */
    this.isActionEditable = function(){
        return m_bActionEditable;
    };
    
    /** Set editable value
     * @param {Boolean} bEditable Boolean value
     */
    this.setValueEditable = function(bEditable){
        m_bValueEditable = bEditable;
    };
    
    /** Get editable
     * @returns {Boolean} bIsEditable Boolean value for describe if value field is editable or not
     */
    this.isValueEditable = function(){
        return m_bValueEditable;
    };
    
    /** Get default value
     * @returns {Integer} Default value
     */
    this.getDefaultValue = function(){
        return m_nValue;
    };
    
    /** Get min
     * @returns {Integer}
     */
    this.getMin = function(){
        return m_nMin;
    };
    
    /** Get max
     * @returns {Integer}
     */
    this.getMax = function(){
        return m_nMax;
    };
    
    
    this.getDescriptions = function(){
        console.log("Descriptions requested");
        var oMap=new Object();
        for (var i=0;i<m_ADescriptions.length;i++)
        {
            oMap[m_ADescriptions[i].value]=m_ADescriptions[i].description;
        }
        console.log(oMap);
        return oMap;
    };
    
    this.resetDescriptions = function(){

        m_ADescriptions=new Array();
    };
    
    this.encodeJSON = function(){
        //Set instance variables


        var oSerialized = {
            editable: m_bEditable,
            value: m_nValue,
            action: m_iAction,
            min: m_nMin,
            max: m_nMax,
            type: m_iType,
            duration: m_iDuration,
            descriptions: this.getDescriptions()
        };
        
        return oSerialized;
    };

};