﻿/** Resources Manager: Business logic singleton class
 *
 * @class ResourcesManager
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 * 
 */

//"use strict";

function ResourcesManager(){
	//Build singleton instance
	if ( arguments.callee._singletonInstance ){
        console.warn("Singleton instance used");
        return arguments.callee._singletonInstance;
    }
        
    arguments.callee._singletonInstance = this;
    
    
	/** Instance variables (private)
	 * 
	 */
	var self = this;
	
	/** Initialized method
     * 
     */
    var init = function(){
        
    };
	
	/** Interface building selected element
	 * @private
	 * @type Node
	 * @default undefined
	 */
	var m_refSelUIComponent = undefined;
	
	
	/** Interface tab root node reference (tree structure)
	 * @private
	 * @type Node
	 * @default new Node({ id: "level-1", m_Data:{  name: "root" }});
	 */
	var m_oSprites = new Array();
	
	
	/** Application's version
     * @public
     * @type String
     */
    this.version = undefined;
    
    /** Application's name
     * @public
     * @type String
     */
    this.name = undefined;
	
	/** Interface html element id generator
     * @public
	 * @type EveIdAssign
	 */
	this.idAssigner = new EveIdAssign("app/resource/");
    
    
    this.spriteDictionary=new Array();
    /**
     * 
     */
    this.addSprite = function(oParams){
        //Check
        //--console.log("add sprite");
       // console.log(oParams);
        if(!oParams["name"]){
            throw "Espect name";
            return;
        }
        else if(!oParams["src"]){
            throw "Espect source";
            return;
        }
        else if(!oParams["iconsHeight"]){
            throw "Espect icons height";
            return;
        }
        else if(!oParams["iconsWidth"]){
            throw "Espect icons width";
            return;
        }

        else if(!oParams["id"]){
            throw "Espected icon Map ID";
            return;
        }


        var oSprite = {
            id: self.idAssigner.getId(),
            params: oParams
        };
        
        m_oSprites.push(oSprite);
        
        //Calculate positions
        oSprite["params"]["_positions"] = new Array();
        // Dario: Added to speedup search	
        self.spriteDictionary[oSprite["id"]]=oSprite;
        var xLimit = oParams["imgWidth"] / oParams["iconsWidth"];
        var i_x = 0;    //Matrix index of x
        var i_y = 0;    //Matrix index of y
        console.log("Resizing in progress in resource manager");
        for(var i=0; i<( oParams["imgWidth"]/oParams["iconsWidth"] * oParams["imgHeight"]/oParams["iconsHeight"] ); ++i){
            
            if(i_x >= xLimit){
                i_x = 0;
                i_y++;
            }
            
            var x = i_x * oParams["iconsWidth"] * (-1);
            var y = i_y * oParams["iconsHeight"] * (-1);
            
            oSprite["params"]["_positions"].push({
                "index": i,
                "class": oSprite["id"],
                "x": x,
                "y": y
            });
            
            i_x++;
           // alert(oSprite["id"]);
          	
        }
        
        return oSprite;
    };
    
    /**
     * 
     */
    this.getSpriteById = function(id){
    	// Dario: Added to speedup search	
    	//console.log("get sprite by id "+id);
    	//console.log(self.spriteDictionary[id]);
    	//return self.spriteDictionary[id];

        if (id=="0") id="app/resource/"+id;
        if (id=="1") id="app/resource/"+id;

        for(var i=0; i<m_oSprites.length; ++i){
            if(m_oSprites[i].id == id){
            //    console.log(m_oSprites[i]);
                return m_oSprites[i];
            }
        }

        return false;

    };
    
    /**
     * 
     */
    this.getSpritesByType = function(type){
        //--console.log("get sprite by type");
        var aResp = new Array();
        
        for(var i=0; i<m_oSprites.length; ++i){
            if(m_oSprites[i]["params"]["type"] == type){
                aResp.push(m_oSprites[i]);
            }
        }
        
        return aResp;
    };
    
	
	/** Initialize Model */
	init();
}